/// 手机端选择器

import 'package:fis_ui/define.dart';
import 'package:flutter/material.dart';

class SelectRow extends StatelessWidget implements FWidget {
  const SelectRow({
    Key? key,
    required this.title,
    this.icon,
    this.isImage = false,
    this.onClick,
    this.hintText,
    this.value = '',
    this.imageUrl = '',
    this.borderText = '',
    this.suffixIcon,
    this.isRequired = false,
    this.fontSize = 14,
  }) : super(key: key);
  final String title;
  final Widget? icon;
  final Widget? suffixIcon;
  final bool? isImage;
  final bool isRequired;
  final String? hintText;
  final Function? onClick;
  final String? value;
  final String? imageUrl;
  final String? borderText;
  final double? fontSize;

  Widget _buildTextContent() {
    if (value!.isEmpty)
      return Text(
        hintText!,
        style: TextStyle(
          fontSize: fontSize,
          color: Color.fromRGBO(155, 155, 155, 1),
        ),
      );
    else
      return Text(
        value!,
        style: TextStyle(
          fontSize: fontSize,
          color: Colors.black,
        ),
        textAlign: TextAlign.right,
        overflow: TextOverflow.ellipsis,
        softWrap: true,
        maxLines: 1,
      );
  }

  Widget _buildContent() {
    if (isImage!) if (imageUrl!.isEmpty)
      return Container(
        width: 40,
        height: 40,
        decoration: BoxDecoration(
          borderRadius: BorderRadius.all(
            Radius.circular(4),
          ),
          border: Border.all(
            color: Color.fromRGBO(211, 211, 211, 1),
            width: 1,
          ),
        ),
        // color: Colors.amberAccent,
        child: Center(
          child: Text(
            (borderText == null || borderText!.isEmpty) ? 'LOGO' : borderText!,
            style: TextStyle(
              color: Color.fromRGBO(211, 211, 211, 1),
              fontSize: fontSize,
            ),
          ),
        ),
      );
    else
      return Container(
        width: 40,
        height: 40,
        decoration: BoxDecoration(
          borderRadius: BorderRadius.all(
            Radius.circular(4),
          ),
          border: Border.all(
            color: Color.fromRGBO(211, 211, 211, 1),
            width: 1,
          ),
        ),
        // color: Colors.amberAccent,
        child: Center(
          child: Image(
            repeat: ImageRepeat.repeat,
            fit: BoxFit.fill,
            image: NetworkImage(imageUrl!),
          ),
        ),
      );
    else
      return _buildTextContent();
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      color: Colors.white,
      child: Ink(
        child: InkWell(
          onTap: () {
            if (onClick != null) {
              onClick!();
            }
          },
          child: Column(
            children: [
              Container(
                padding: EdgeInsets.symmetric(horizontal: 20),
                height: 50,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Container(
                      child: Row(
                        children: [
                          Row(
                            children: [
                              icon ?? Text(''),
                            ],
                          ),
                          Text.rich(
                            TextSpan(children: [
                              TextSpan(
                                  text: title,
                                  style: TextStyle(fontSize: fontSize)),
                              if (isRequired) ...[
                                TextSpan(
                                  text: ' *',
                                  style: TextStyle(color: Colors.red),
                                )
                              ] else ...[
                                TextSpan(text: '')
                              ]
                            ]),
                          ),
                        ],
                      ),
                    ),
                    Container(
                      child: Row(
                        children: [
                          Container(
                            alignment: Alignment.centerRight,
                            child: _buildContent(),
                          ),
                          suffixIcon ??
                              Icon(
                                Icons.chevron_right,
                                color: Color.fromRGBO(164, 164, 164, 1),
                              ),
                        ],
                      ),
                    )
                  ],
                ),
              ),
              Container(
                margin: EdgeInsets.symmetric(horizontal: 20),
                decoration: BoxDecoration(
                  border: Border(
                    bottom: BorderSide(
                      color: Color.fromRGBO(245, 245, 245, 1),
                    ),
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
