/// 多选框的初步封装

import 'package:fis_ui/define.dart';
import 'package:flutter/material.dart';

class FCheckbox<T> extends StatefulWidget implements FWidget {
  const FCheckbox({
    Key? key,
    this.onChange,
    required this.value,
    this.isCheck = false,
    this.materialTapTargetSize,
    this.activeColor,
    this.visualDensity,
    this.onChanged,
    this.fillColor,
    this.checkColor,
  }) : super(key: key);
  final void Function(T value, bool? isCheck)? onChange;
  final T value;
  final bool? isCheck;

  /// Configures the minimum size of the tap target.
  final MaterialTapTargetSize? materialTapTargetSize;

  /// The color to use when this checkbox is checked.
  final Color? activeColor;

  /// Defines how compact the checkbox's layout will be.
  final VisualDensity? visualDensity;

  /// Called when the value of the checkbox should change.
  final ValueChanged<bool?>? onChanged;

  /// The color that fills the checkbox, in all [MaterialState]s.
  /// ///
  /// Resolves in the following states:
  ///  * [MaterialState.selected].
  ///  * [MaterialState.hovered].
  ///  * [MaterialState.focused].
  ///  * [MaterialState.disabled].
  ///
  final MaterialStateProperty<Color?>? fillColor;

  /// {@template flutter.material.checkbox.checkColor}
  /// The color to use for the check icon when this checkbox is checked.
  /// {@endtemplate}
  ///
  /// If null, then the value of [CheckboxThemeData.checkColor] is used. If
  /// that is also null, then Color(0xFFFFFFFF) is used.
  final Color? checkColor;
  @override
  State<FCheckbox> createState() => _FcheckboxState();
}

class _FcheckboxState<T> extends State<FCheckbox<T>> {
  bool isClick = false;

  @override
  void initState() {
    isClick = widget.isCheck ?? false;
    super.initState();
  }

  Widget _buildChild() {
    var childs;
    isClick
        ? childs = Container(
            decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(15),
              color: Theme.of(context).primaryColor,
            ),
            child: Center(
              child: Container(
                child: Icon(
                  Icons.check,
                  color: Colors.white,
                ),
              ),
            ),
          )
        : childs = Text('');
    return childs;
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        setState(() {
          isClick = !isClick;
        });
        widget.onChange?.call(widget.value, isClick);
      },
      child: Container(
        width: 30,
        height: 30,
        decoration: BoxDecoration(
          border: Border.all(
            color: Color.fromRGBO(141, 141, 141, 1),
            width: 0.5,
          ),
          borderRadius: BorderRadius.circular(30),
        ),
        child: _buildChild(),
      ),
    );
  }
}
