import 'package:fis_ui/interface/active_widget.dart';
import 'package:fis_ui/interface/interactive_container.dart';
import 'package:flutter/material.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/services.dart';

class FCounter extends FStatefulWidget implements FActiveWidget {
  FCounter({
    required this.businessParent,
    required this.name,
    required this.onChange,
    required this.textController,
    this.initValue = 1.0,
    this.step = 1,
    Key? key,
  }) : super(key: key);
  @override
  final FInteractiveContainer businessParent;

  final double? initValue;
  final ValueCallback<double?> onChange;
  final double step;
  final TextEditingController textController;

  @override
  final String name;

  @override
  FState<FCounter> createState() => _FCounterState();
}

class _FCounterState extends FState<FCounter> {
  double _counter = 1.0;

  static const C_BOX_SIZD = 33.0;
  static const C_BORDER_COLOR = Color.fromRGBO(141, 141, 141, 1);
  static const C_BORDER_RADIUS = 4.0;
  @override
  void initState() {
    super.initState();

    _counter = widget.initValue!;
  }

  @override
  void didUpdateWidget(covariant StatefulWidget oldWidget) {
    _counter = widget.initValue!;
    super.didUpdateWidget(oldWidget);
  }

  @override
  FWidget build(BuildContext context) {
    return FRow(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        _buildIconButton(
          Icons.remove,
          () {
            setState(() {
              _counter -= widget.step;
              widget.textController.text = _counter.toString();
            });
            widget.onChange(_counter);
          },
          true,
        ),
        FContainer(
          height: C_BOX_SIZD,
          width: 80,
          // 让TextField实现上下居中，
          alignment: Alignment.center,
          decoration: const BoxDecoration(
            //上下边框
            border: Border.symmetric(
              horizontal: BorderSide(
                color: C_BORDER_COLOR,
                width: 0.5,
              ),
            ),
          ),
          child: FContainer(
            child: FTextField(
              onChanged: (value) {
                _counter = double.parse(value);
                widget.onChange(_counter);
              },
              controller: widget.textController,
              inputFormatters: [
                FilteringTextInputFormatter.allow(
                  new RegExp(
                    r'[0-9]',
                  ),
                ),
              ],
              // 让TextField内容实现左右居中，
              textAlign: TextAlign.center,
              decoration: null,
            ),
          ),
        ),
        _buildIconButton(
          Icons.add,
          () {
            setState(
              () {
                _counter += widget.step;
                widget.textController.text = _counter.toString();
              },
            );
            widget.onChange(_counter);
          },
          false,
        ),
      ],
    );
  }

  FWidget _buildIconButton(IconData icon, Function onclick, bool isLeft) {
    return FSizedBox(
      height: C_BOX_SIZD,
      width: C_BOX_SIZD,
      child: FOutlinedButton(
        style: FOutlinedButton.styleFrom(
          shadowColor: Colors.transparent,
          side: BorderSide(
            width: 0.5,
            color: C_BORDER_COLOR,
          ),
          //圆角
          shape: RoundedRectangleBorder(
            borderRadius: isLeft
                ? BorderRadius.only(
                    topLeft: Radius.circular(
                      C_BORDER_RADIUS,
                    ),
                    bottomLeft: Radius.circular(
                      C_BORDER_RADIUS,
                    ),
                  )
                : BorderRadius.only(
                    topRight: Radius.circular(
                      C_BORDER_RADIUS,
                    ),
                    bottomRight: Radius.circular(
                      C_BORDER_RADIUS,
                    ),
                  ),
          ),
          padding: EdgeInsets.symmetric(
            horizontal: 0,
          ),
        ),
        onPressed: () {
          onclick();
        },
        businessParent: widget.businessParent,
        name: '',
        child: FIcon(icon),
      ),
    );
  }
}
