import 'package:fis_ui/base_define/page.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';

/// 待优化
class FDateTimeInput extends FStatefulWidget implements FPage {
  const FDateTimeInput({
    super.key,
    required this.currentKey,
    required this.textEditingController,
    required this.initialDate,
    required this.onChangeTime,
  });
  final GlobalKey currentKey;
  final TextEditingController textEditingController;
  final DateTime initialDate;
  final ValueCallback onChangeTime;

  @override
  FState<FDateTimeInput> createState() => _FDateTimeInputState();

  @override
  String get pageName => 'FDateTimeInput';
}

class _FDateTimeInputState extends FState<FDateTimeInput> {
  @override
  FWidget build(BuildContext context) {
    return _buildTimeInput(
      context: context,
      currentKey: widget.currentKey,
      textEditingController: widget.textEditingController,
      onChangeTime: (value) => widget.onChangeTime(value),
    );
  }

  FWidget _buildTimeInput({
    required BuildContext context,
    required GlobalKey currentKey,
    String? hintText,
    required TextEditingController textEditingController,
    required ValueCallback onChangeTime,
  }) {
    DateTime initialDate;
    textEditingController.text =
        DateFormat('yyyy-MM-dd').format(widget.initialDate);
    return FBorderInput(
      key: currentKey,
      hintSize: 16,
      contentSize: 16,
      maxLength: 20,
      borderColor: Color.fromRGBO(226, 228, 234, 1),
      controller: textEditingController,
      readOnly: true,
      onTap: () {
        initialDate = textEditingController.text.isNotEmpty
            ? DateTime.parse(textEditingController.text)
            : DateTime.now();
        _builTimeDialog(
          currentKey,
          textEditingController,
          initialDate,
        );
      },
      suffixIcon: FMaterial(
        color: Colors.transparent,
        child: FIconButton(
          name: "startTimePosition",
          businessParent: this.widget,
          onPressed: () {
            initialDate = widget.initialDate;
            _builTimeDialog(
              currentKey,
              textEditingController,
              initialDate,
            );
          },
          icon: FIcon(
            Icons.date_range_rounded,
          ),
        ),
      ),
      height: 36,
      hintText: hintText,
    );
  }

  _findRenderObject(GlobalKey key) {
    RenderBox? renderBox = key.currentContext!.findRenderObject() as RenderBox;
    // offset.dx , offset.dy 就是控件的左上角坐标
    renderBox.localToGlobal(Offset.zero);
    var offset = renderBox.localToGlobal(Offset.zero);
    return [offset.dx, offset.dy + 38];
  }

  void _builTimeDialog(
    GlobalKey<State<StatefulWidget>> currentKey,
    TextEditingController textEditingController,
    DateTime? initialDate,
  ) {
    var timePosition = _findRenderObject(
      currentKey,
    );
    showDialog(
      context: context,
      barrierDismissible: false,
      builder: (BuildContext context) {
        return FTimeDialog(
          position: timePosition,
          globalKey: currentKey.toString(),
          timeCallback: (dateTime) {
            initialDate = dateTime;
            widget.onChangeTime.call(dateTime);
            textEditingController.text =
                DateFormat('yyyy-MM-dd').format(dateTime);
          },
          initialDate: initialDate,
        );
      },
    );
  }
}

class FTimeDialog extends Dialog implements FPage {
  final ValueCallback timeCallback;
  final List? position;
  final String? globalKey;
  final DateTime? initialDate;
  @override
  final String pageName;

  FTimeDialog({
    Key? key,

    /// 数据回调
    required this.timeCallback,

    /// 弹框出现位置
    this.position,

    /// 初始日期
    this.initialDate,

    /// 当前key
    this.globalKey,
    this.pageName = 'ProductListDialog',
  }) : super(key: key);
  @override
  FWidget build(BuildContext context) {
    return FMaterial(
      type: MaterialType.transparency,
      child: ProductListView(context, globalKey),
    );
  }

  FWidget ProductListView(context, globalKey) => FSafeArea(
        child: FGestureDetector(
          name: "back",
          businessParent: this,
          child: FStack(
            children: [
              FContainer(
                width: MediaQuery.of(context).size.width,
                height: MediaQuery.of(context).size.height,
                color: Colors.transparent, // 它的颜色为透明色
              ),
              FPositioned(
                top: position![1],
                left: position![0],
                child: FContainer(
                  constraints: BoxConstraints(
                    maxWidth: 310,
                    maxHeight: 400,
                  ),
                  decoration: BoxDecoration(
                    color: Colors.white,
                    boxShadow: [
                      BoxShadow(
                        blurRadius: 5,
                        color: Color(0xffeeeeee),
                      )
                    ],
                    borderRadius: BorderRadius.circular(5),
                  ),
                  child: _calendarDatePicker(
                    DatePickerMode.day,
                    context,
                    globalKey,
                    initialDate,
                  ),
                ),
              ),
            ],
          ),
          onTap: () {
            Navigator.of(context).pop();
          },
        ),
      );
  FCalendarDatePicker _calendarDatePicker(
    DatePickerMode mode,
    BuildContext context,
    String buildKey,
    DateTime? initialDate,
  ) {
    return FCalendarDatePicker(
      initialDate: initialDate ?? DateTime.now(),
      firstDate: DateTime(1996, 10, 18), // 开始日期
      lastDate: DateTime.now(), // 结束日期
      initialCalendarMode: mode, // 日期选择样式
      // 选中日期改变回调函数
      onDateChanged: (dateTime) {
        initialDate = dateTime;
        Navigator.of(context).pop();
        timeCallback.call(dateTime);
      },
      // 月份改变回调函数
      onDisplayedMonthChanged: (dateTime) {},
      // 筛选日期可不可点回调函数
      selectableDayPredicate: (dayTime) {
        if (dayTime == DateTime(2020, 10, 15) ||
            dayTime == DateTime(2020, 10, 16) ||
            dayTime == DateTime(2020, 10, 17)) {
          return false;
        }
        return true;
      },
    );
  }
}
