/// 手机端输入文本框的组件
import 'package:fis_ui/define.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class InputRow extends StatelessWidget implements FWidget {
  const InputRow({
    Key? key,
    required this.title,
    this.icon,
    this.isSelect = false,
    this.onClick,
    this.hintText = '请输入',
    this.controller,
    this.onChanged,
    this.readOnly,
    this.maxLength,
    this.inputFormatters,
    this.maxLines = 1,
  }) : super(key: key);
  final String title;
  final Widget? icon;
  final bool? isSelect;
  final Function? onClick;
  final String? hintText;
  final TextEditingController? controller;
  final void Function(String)? onChanged;
  final bool? readOnly;
  final List<TextInputFormatter>? inputFormatters;
  final int? maxLength;
  final int? maxLines;

  @override
  Widget build(BuildContext context) {
    return Material(
      color: Colors.white,
      child: Column(
        children: [
          Container(
            padding: EdgeInsets.symmetric(horizontal: 20),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Container(
                  child: Row(
                    children: [
                      Row(
                        children: [
                          icon ?? Text(''),
                        ],
                      ),
                      Text(title),
                    ],
                  ),
                ),
                Container(
                  child: Row(
                    children: [
                      Container(
                        width: 180,
                        padding: EdgeInsets.only(right: 20),
                        child: TextField(
                          maxLengthEnforcement:
                              MaxLengthEnforcement.truncateAfterCompositionEnds,
                          maxLines: maxLines,
                          textAlign: TextAlign.end,
                          maxLength: maxLength,
                          style: TextStyle(
                              color: readOnly ?? false
                                  ? Colors.grey
                                  : Colors.black,
                              fontSize: 14),
                          controller: controller,
                          onChanged: onChanged,
                          readOnly: readOnly ?? false,
                          inputFormatters: inputFormatters,
                          decoration: InputDecoration(
                            hintText: hintText!,
                            hintStyle: TextStyle(
                              fontSize: 14,
                              color: Color.fromRGBO(155, 155, 155, 1),
                            ),
                            enabledBorder: InputBorder.none,
                            focusedBorder: InputBorder.none,
                            counterText: '',
                          ),
                        ),
                      ),
                    ],
                  ),
                )
              ],
            ),
          ),
          Container(
            margin: EdgeInsets.symmetric(horizontal: 20),
            decoration: BoxDecoration(
              border: Border(
                bottom: BorderSide(
                  color: Color.fromRGBO(245, 245, 245, 1),
                ),
              ),
              // color: Colors.white,
            ),
          ),
        ],
      ),
    );
  }
}
