// import 'package:fis_lib_dropdown_button/dropdown_button2.dart'
//     as fis_lib_dropdown;

import 'dart:async';

import 'package:fis_common/event/event_type.dart';
import 'package:fis_ui/define.dart';
import 'package:flutter/material.dart';

import 'select/dropdown_button2.dart';

class FSelectOptionModel<T> {
  FSelectOptionModel({
    required this.value,
    required this.title,
  });

  static generateList(int count) {
    return List.generate(
      count,
      (index) => FSelectOptionModel(value: index, title: "User_$index"),
    );
  }

  final T value;
  final String title;
}

/// 单选框
class FSelect<T, TValue> extends StatefulWidget implements FWidget {
  const FSelect({
    Key? key,
    required this.source,
    this.value,
    required this.optionValueExtractor,
    required this.optionLabelExtractor,
    this.onSelectChanged,
    this.hintText,
    this.clearable = false,
    this.isExpanded = false,
    this.width,
    this.height,
    this.fontSize,
    this.decoration,
    this.specialWidget,
    this.specialWidgetHeight,
    this.onLoad,
    this.isSearchInner = false,
    this.textColor,
    this.fontFamily,
    this.buttonDecoration,
    this.focusNode,
    this.onReload,
    this.dropDownIcon,
    this.optionAlignment = Alignment.centerLeft,
  }) : super(key: key);

  final Widget? dropDownIcon;

  ///按钮装饰器
  final BoxDecoration? buttonDecoration;

  ///文本颜色
  final Color? textColor;

  ///文本字体
  final String? fontFamily;

  /// 数据源
  final List<T> source;

  /// 已选值
  final TValue? value;

  /// 选项值提取器
  ///
  /// [data] 选项数据源
  final TValue Function(T data) optionValueExtractor;

  /// 选项文本提取器
  ///
  /// [data] 选项数据源
  final String Function(T data) optionLabelExtractor;

  /// 选中变更事件
  ///
  /// [value] 当前已选值
  ///
  /// [index] 当前已选索引
  final void Function(TValue? value, int? index)? onSelectChanged;

  /// 选择提示文本
  final String? hintText;

  /// 是否可清除
  final bool clearable;

  /// 是否自适应 (开启后[width]和[height]不起作用)
  final bool isExpanded;

  /// 自定义宽度
  final double? width;

  /// 自定义高度
  final double? height;

  /// 自定义字体大小
  final double? fontSize;

  /// 自定义item
  final Widget Function(T data)? specialWidget;

  /// 自定义的item的高度
  final double? specialWidgetHeight;

  /// 装饰器
  final BoxDecoration? decoration;

  /// 刷新加载
  final Function(String? keyWords)? onLoad;

  /// 是否展示搜索功能
  final bool? isSearchInner;

  final FocusNode? focusNode;

  final FEventHandler<bool>? onReload;

  final Alignment optionAlignment;

  @override
  State<StatefulWidget> createState() => _FSelectState<T, TValue>();
}

class _FSelectState<T, TValue> extends State<FSelect<T, TValue>> {
  TValue? _value;

  /// 数据源
  List<T> _source = [];
  final TextEditingController textEditingController = TextEditingController();

  @override
  void initState() {
    _value = widget.value;
    _source = widget.source;
    super.initState();
    textEditingController.addListener(() {
      buildItems();
      setState(() {});
    });
  }

  @override
  void didUpdateWidget(covariant FSelect<T, TValue> oldWidget) {
    _value = widget.value;
    if (_source != widget.source) {
      setState(() {
        _source = widget.source;
      });
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    final buttonWidth = widget.isExpanded ? null : widget.width ?? 200;
    final buttonHeight = widget.isExpanded ? null : widget.height ?? 44;
    // 适应更小号文字
    final hint = Text(
      widget.hintText ?? "Select",
      style: TextStyle(
          fontSize: widget.fontSize ?? 16, color: Colors.grey, height: 1),
    );

    final icon = widget.clearable && _value != null
        ? GestureDetector(
            onTap: () {
              handleChange(null);
            },
            child: const Icon(Icons.close_rounded, size: 20),
          )
        : widget.dropDownIcon;
    const deFaultDurationTime = 300;
    Timer? timer;

    var buttonDecoration = widget.buttonDecoration ??
        BoxDecoration(
          border: Border.all(color: Color(0xffdcdfe6), width: 0.8),
          // border: Border.all(color: Colors.grey[200]!, width: 0.5),
          borderRadius: BorderRadius.circular(4),
        );
    if (widget.decoration != null) {
      final cfg = widget.decoration!;
      buttonDecoration = widget.buttonDecoration ??
          buttonDecoration.copyWith(
            color: cfg.color,
            image: cfg.image,
            border: cfg.border,
            borderRadius: cfg.borderRadius,
            backgroundBlendMode: cfg.backgroundBlendMode,
            boxShadow: cfg.boxShadow,
            shape: cfg.shape,
            gradient: cfg.gradient,
          );
    }

    /// 节流函数 处理规定在一个单位时间内,只能触发一次函数。如果这个单位时间内触发多次函数,只有一次生效
    const String deFaultThrottleId = 'DeFaultThrottleId';
    Map<String, int> startTimeMap = {deFaultThrottleId: 0};
    void throttle(Function doSomething,
        {String throttleId = deFaultThrottleId,
        durationTime = deFaultDurationTime,
        Function? continueClick}) {
      int currentTime = DateTime.now().millisecondsSinceEpoch;
      if (currentTime - (startTimeMap[throttleId] ?? 0) > durationTime) {
        doSomething.call();
        startTimeMap[throttleId] = DateTime.now().millisecondsSinceEpoch;
      } else {
        continueClick?.call();
      }
    }

    List<Widget> selectedItemBuilder() {
      return _source.map((e) {
        return _OptionRow(
          widget.optionLabelExtractor(e),
          optionAlignment: widget.optionAlignment,
          height: buttonHeight,
          fontFamily: widget.fontFamily,
          fontSize: widget.fontSize,
        );
      }).toList();
    }

    return DropdownButtonHideUnderline(
      child: DropdownButton2<TValue>(
        hint: hint,
        icon: icon,
        onReload: widget.onReload,
        focusNode: widget.focusNode,
        dropdownMaxHeight: 280,
        buttonWidth: buttonWidth,
        buttonHeight: buttonHeight,
        itemHeight:
            widget.specialWidget != null ? widget.specialWidgetHeight! : 38,
        itemPadding: const EdgeInsets.symmetric(horizontal: 14, vertical: 2),
        isExpanded: widget.isExpanded,
        dropdownOverButton: false,
        scrollbarAlwaysShow: true,
        scrollbarThickness: 4,
        scrollbarRadius: const Radius.circular(2),
        buttonPadding: const EdgeInsets.symmetric(horizontal: 14, vertical: 4),
        dropdownDecoration: BoxDecoration(
          borderRadius: BorderRadius.circular(4),
        ),
        style:
            TextStyle(color: widget.textColor, fontFamily: widget.fontFamily),
        buttonDecoration: buttonDecoration,
        offset: const Offset(0, -4),
        selectedItemHighlightColor: Theme.of(context).secondaryHeaderColor,
        selectedItemBuilder: (_) => selectedItemBuilder(),
        value: _value,
        items: buildItems(),
        onChanged: handleChange,
        searchController: textEditingController,
        searchInnerWidget: widget.isSearchInner!
            ? Padding(
                padding: const EdgeInsets.only(
                  top: 8,
                  bottom: 4,
                  right: 8,
                  left: 8,
                ),
                child: TextFormField(
                  controller: textEditingController,
                  decoration: InputDecoration(
                    isDense: true,
                    contentPadding: const EdgeInsets.symmetric(
                      horizontal: 10,
                      vertical: 8,
                    ),
                    hintText: '',
                    hintStyle: const TextStyle(fontSize: 12),
                    border: OutlineInputBorder(
                      borderRadius: BorderRadius.circular(8),
                    ),
                  ),
                ),
              )
            : Container(),
        searchMatchFn: (item, searchValue) {
          final index = _source.indexWhere(
              (x) => widget.optionValueExtractor(x) == item.value.toString());
          if (searchValue.isEmpty)
            return true;
          else
            return widget
                .optionLabelExtractor(_source[index])
                .toString()
                .contains(textEditingController.text);
        },
        //This to clear the search value when you close the menu
        onMenuStateChange: (isOpen) {
          if (!isOpen) {
            textEditingController.clear();
          }
        },
      ),
    );
  }

  void handleChange(TValue? value) {
    setState(() {
      _value = value;
    });
    if (widget.onSelectChanged != null) {
      if (value == null) {
        widget.onSelectChanged!.call(null, null);
      } else {
        final index = _source.indexWhere(
          (x) => widget.optionValueExtractor(x) == value,
        );
        widget.onSelectChanged!.call(value, index);
      }
    }
  }

  List<DropdownMenuItem<TValue>> buildItems() {
    return _source.map((T e) {
      final value = widget.optionValueExtractor(e);
      final active = value == _value;
      if (widget.specialWidget == null) {
        return DropdownMenuItem<TValue>(
          child: _OptionRow(
            widget.optionLabelExtractor(e),
            isActive: active,
            fontSize: widget.fontSize,
            fontFamily: widget.fontFamily,
            buttonDecoration: widget.buttonDecoration,
          ),
          value: value,
        );
      } else {
        return DropdownMenuItem<TValue>(
          child: _OptionRow(
            widget.optionLabelExtractor(e),
            isActive: active,
            fontSize: widget.fontSize,
            fontFamily: widget.fontFamily,
            specialWidget: widget.specialWidget!(e),
            buttonDecoration: widget.buttonDecoration,
          ),
          value: value,
        );
      }
    }).toList();
  }
}

class _OptionRow extends StatelessWidget {
  const _OptionRow(
    this.label, {
    Key? key,
    this.isActive = false,
    this.height,
    this.fontSize,
    this.specialWidget,
    this.fontFamily,
    this.buttonDecoration,
    this.optionAlignment = Alignment.centerLeft,
  }) : super(key: key);

  final String? fontFamily;
  final String label;
  final bool isActive;
  final double? height;
  final double? fontSize;
  final Widget? specialWidget;
  final BoxDecoration? buttonDecoration;
  final Alignment optionAlignment;

  @override
  Widget build(BuildContext context) {
    final text = Text(
      label,
      style: TextStyle(
          overflow: TextOverflow.ellipsis,
          fontSize: fontSize ?? 16,
          fontFamily: fontFamily,
          color: isActive ? Theme.of(context).primaryColor : null,
          height: 1),
    );
    if (specialWidget != null) {
      return specialWidget!;
    } else {
      if (height != null) {
        return Container(
          alignment: optionAlignment,
          height: height,
          decoration: buttonDecoration,
          child: text,
        );
      } else {
        return text;
      }
    }
  }
}

class FSelectModel {
  FSelectModel({required this.name, required this.code});
  final String name;
  final String code;
}
