// https://pub.flutter-io.cn/packages/flutter_typeahead
import 'dart:async';

import 'package:fis_ui/define.dart';
import 'package:fis_ui/special/flutter_typeahead/flutter_typeahead.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'input.dart';
import '../types/data_builders.dart';
import '../types/widget_builders.dart';

/// 带建议选项输入框
class FSuggesionInput<T> extends StatelessWidget implements FWidget {
  const FSuggesionInput({
    Key? key,
    required this.optionItemBuilder,
    required this.optionItemsFetcher,
    required this.onOptionSelected,
    required this.noItemsFoundText,
    this.onLoadMore,
    this.keepSuggestionsOnSuggestionSelected = false,
    this.debounceDuration = const Duration(milliseconds: 300),
    this.inputFieldConfiguration,
    this.hideOnEmpty = false,
    this.hideOnLoading = false,
  }) : super(key: key);

  /// 可选项组件建造器
  final DataWidgetBuilder<T> optionItemBuilder;

  /// 建议数据获取方法
  final DataListBuilderWithParameter<T, String> optionItemsFetcher;

  /// 可选项选中回调
  final ValueCallback<T> onOptionSelected;

  /// 防抖限时
  final Duration debounceDuration;

  final FSuggesionInputFieldConfiguration? inputFieldConfiguration;

  /// 没有可选项时的提示文本
  final String noItemsFoundText;

  /// 隐藏载入动画
  final bool hideOnLoading;

  /// 无匹配项时隐藏建议
  final bool hideOnEmpty;

  /// 列表项选中时保持列表展开
  final bool keepSuggestionsOnSuggestionSelected;

  ///自定义的加载更多的回调
  final FutureOr<Iterable<T>> Function(String)? onLoadMore;

  @override
  Widget build(BuildContext context) {
    final c = inputFieldConfiguration;
    return TypeAheadField<T>(
      itemBuilder: (context, itemData) => optionItemBuilder(context, itemData),
      onSuggestionSelected: (T suggestion) => onOptionSelected(suggestion),
      suggestionsCallback: (String pattern) => optionItemsFetcher(pattern),
      onLoadMore: onLoadMore,
      noItemsFoundBuilder: (context) => ListTile(
        title: Text(
          noItemsFoundText,
          style: const TextStyle(
            color: Colors.grey,
            fontSize: FInput.hintSize,
          ),
        ),
      ),
      debounceDuration: debounceDuration,
      hideOnLoading: hideOnLoading,
      hideOnEmpty: hideOnEmpty,
      keepSuggestionsOnSuggestionSelected: keepSuggestionsOnSuggestionSelected,
      textFieldConfiguration: TextFieldConfiguration(
        decoration: c?.decoration ?? FInput.borderDecoration(),
        inputFormatters: c?.inputFormatters,
        onChanged: c?.onChanged,
        controller: c?.controller,
        focusNode: c?.focusNode,
        enabled: c?.enabled == true,
        autofocus: c?.autofocus == true,
        maxLength: c?.maxLength,
        style: c?.style ??
            TextStyle(
              color: FInput.txtColor,
              fontSize: FInput.contentSize,
            ),
      ),
    );
  }
}

/// 输入框选项 同 TextField
class FSuggesionInputFieldConfiguration {
  FSuggesionInputFieldConfiguration({
    this.inputFormatters,
    this.controller,
    this.focusNode,
    this.enabled = true,
    this.autofocus = false,
    this.maxLength,
    this.onChanged,
    this.decoration,
    this.style,
  });
  final List<TextInputFormatter>? inputFormatters;
  final TextEditingController? controller;
  final FocusNode? focusNode;
  final bool enabled;
  final bool autofocus;
  final int? maxLength;
  final ValueChanged<String>? onChanged;
  final InputDecoration? decoration;
  final TextStyle? style;
}
