import 'package:fis_common/helpers/color.dart';
import 'package:fis_common/logger/logger.dart';
import 'package:fis_theme/theme.dart';
import 'package:flutter/material.dart';

import '../define.dart';

class SwipeCard extends StatefulWidget implements FWidget {
  final Widget child;
  final int? index;
  final Function onTap;
  final bool isActive;
  final EdgeInsets forbiddenAnimationMargin;
  final EdgeInsets forbiddenAnimationPadding;

  /// 是否禁用动画
  final bool? isForbiddenAnimation;

  const SwipeCard({
    Key? key,
    this.index,
    required this.child,
    required this.onTap,
    required this.isActive,
    this.forbiddenAnimationMargin = const EdgeInsets.only(
      right: 22,
      left: 22,
    ),
    this.forbiddenAnimationPadding = const EdgeInsets.only(
      top: 20,
      bottom: 10,
      left: 15,
      right: 0,
    ),
    this.isForbiddenAnimation = false,
  }) : super(key: key);

  @override
  State<SwipeCard> createState() => _SwipeCardState();
}

class _SwipeCardState extends State<SwipeCard> {
  bool hovering = false;

  Widget _buildChild() {
    EdgeInsets margin = EdgeInsets.only(
      right: 22,
      left: 22,
    );

    Color color = Colors.white;
    BoxShadow boxShadow = BoxShadow(
      color: FColorHelper.mixColor(
        Colors.black12,
        Colors.black38,
        70,
      ),
      offset: Offset(3.0, 3.0), //阴影y轴偏移量
      blurRadius: 6, //阴影模糊程度
      spreadRadius: 0, //阴影扩散程度
    );
    EdgeInsets padding =
        EdgeInsets.only(top: 20, bottom: 10, left: 15, right: 0);

    if (hovering && !widget.isActive) {
      margin = EdgeInsets.only(
        right: 15,
        left: 15,
      );
      color = FTheme.ins.colorScheme.secondary;
      boxShadow = BoxShadow(
        color: FTheme.ins.colorScheme.secondary,
        offset: Offset(3.0, 3.0), //阴影y轴偏移量
        blurRadius: 6, //阴影模糊程度
        spreadRadius: 0, //阴影扩散程度
      );
      padding = EdgeInsets.only(
        top: 20,
        bottom: 10,
        left: 22,
        right: 0,
      );
    } else if (widget.isActive) {
      color = FTheme.ins.colorScheme.primary;
      boxShadow = BoxShadow(
        color: FColorHelper.mixColor(
          FTheme.ins.colorScheme.primary,
          Colors.black,
          70,
        ),
        offset: Offset(3.0, 3.0), //阴影y轴偏移量
        blurRadius: 6, //阴影模糊程度
        spreadRadius: 0, //阴影扩散程度
      );
    }
    return AnimatedContainer(
      duration: const Duration(milliseconds: 200),
      decoration: BoxDecoration(
        color: color,
        borderRadius: BorderRadius.all(
          Radius.circular(8),
        ),
        boxShadow: [
          boxShadow,
        ],
      ),
      margin: widget.isForbiddenAnimation!
          ? widget.forbiddenAnimationMargin
          : margin,
      padding: widget.isForbiddenAnimation!
          ? widget.forbiddenAnimationPadding
          : padding,
      child: widget.child,
    );
  }

  @override
  Widget build(BuildContext context) {
    return MouseRegion(
      cursor: SystemMouseCursors.click,
      onEnter: (e) {
        logger.i("SwipeCard onEnter index:${widget.index}");
        setState(() {
          hovering = true;
        });
      },
      onExit: (e) {
        logger.i("SwipeCard onExit index:${widget.index}");
        setState(() {
          hovering = false;
        });
      },
      child: GestureDetector(
        onTap: () {
          logger.i("SwipeCard onTap index:${widget.index}");
          setState(() {
            widget.onTap();
          });
        },
        child: Column(
          children: [
            _buildChild(),
            SizedBox(height: 15),
          ],
        ),
      ),
    );
  }
}
