import 'dart:async';
import 'dart:ui';
import 'dart:math' as math;
import 'package:fis_theme/theme.dart';
import 'package:fis_ui/define.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_ui/interface/active_widget.dart';
import 'package:fis_ui/interface/interactive_container.dart';
import 'package:flutter/material.dart';

/// 按钮(按下时抬高) from Material
class FElevatedButton extends ElevatedButton implements FActiveWidget {
  FElevatedButton({
    Key? key,
    required FWidget child,
    required VoidCallback? onPressed,
    required this.businessParent,
    required this.name,
    ButtonStyle? style,
    VoidCallback? onLongPress,
    ValueChanged<bool>? onHover,
    ValueChanged<bool>? onFocusChange,
    FocusNode? focusNode,
    bool autofocus = false,
    Clip clipBehavior = Clip.none,
  }) : super(
          key: key,
          child: child,
          onPressed: onPressed == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      (name ?? '') + "/" + "onPressed start");
                  onPressed.call();
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      (name ?? '') + "/" + "onPressed end");
                },
          style: style,
          onLongPress: onLongPress == null
              ? null
              : (() {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      (name ?? '') + "/" + "onLongPress start");
                  onLongPress.call();
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      (name ?? '') + "/" + "onLongPress end");
                }),
          onHover: onHover,
          onFocusChange: onFocusChange,
          focusNode: focusNode,
          autofocus: autofocus,
          clipBehavior: clipBehavior,
        );

  factory FElevatedButton.icon({
    Key? key,
    required VoidCallback? onPressed,
    VoidCallback? onLongPress,
    ValueChanged<bool>? onHover,
    ValueChanged<bool>? onFocusChange,
    ButtonStyle? style,
    FocusNode? focusNode,
    bool? autofocus,
    Clip? clipBehavior,
    required Widget icon,
    required Widget label,
    required FInteractiveContainer businessParent,
    required String name,
  }) = FElevatedButtonWithIcon;

  @override
  final FInteractiveContainer businessParent;

  @override
  final String? name;
}

/// 轮廓按钮 from Material
class FOutlinedButton extends OutlinedButton implements FActiveWidget {
  FOutlinedButton({
    Key? key,
    required FWidget child,
    required VoidCallback? onPressed,
    ButtonStyle? style,
    required this.businessParent,
    required this.name,
  }) : super(
          key: key,
          child: child,
          onPressed: onPressed == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, (name ?? '') + " start");
                  onPressed.call();
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, (name ?? '') + " end");
                },
          style: style,
        );

  @override
  final FInteractiveContainer businessParent;

  @override
  final String? name;
  static ButtonStyle styleFrom({
    Color? foregroundColor,
    Color? backgroundColor,
    Color? disabledForegroundColor,
    Color? disabledBackgroundColor,
    Color? shadowColor,
    Color? surfaceTintColor,
    double? elevation,
    TextStyle? textStyle,
    EdgeInsetsGeometry? padding,
    Size? minimumSize,
    Size? fixedSize,
    Size? maximumSize,
    BorderSide? side,
    OutlinedBorder? shape,
    MouseCursor? enabledMouseCursor,
    MouseCursor? disabledMouseCursor,
    VisualDensity? visualDensity,
    MaterialTapTargetSize? tapTargetSize,
    Duration? animationDuration,
    bool? enableFeedback,
    AlignmentGeometry? alignment,
    InteractiveInkFeatureFactory? splashFactory,
    @Deprecated('Use foregroundColor instead. '
        'This feature was deprecated after v3.1.0.')
        Color? primary,
    @Deprecated('Use disabledForegroundColor instead. '
        'This feature was deprecated after v3.1.0.')
        Color? onSurface,
  }) {
    return ButtonStyle(
      textStyle: ButtonStyleButton.allOrNull<TextStyle>(textStyle),
      shadowColor: ButtonStyleButton.allOrNull<Color>(shadowColor),
      surfaceTintColor: ButtonStyleButton.allOrNull<Color>(surfaceTintColor),
      elevation: ButtonStyleButton.allOrNull<double>(elevation),
      padding: ButtonStyleButton.allOrNull<EdgeInsetsGeometry>(padding),
      minimumSize: ButtonStyleButton.allOrNull<Size>(minimumSize),
      fixedSize: ButtonStyleButton.allOrNull<Size>(fixedSize),
      maximumSize: ButtonStyleButton.allOrNull<Size>(maximumSize),
      side: ButtonStyleButton.allOrNull<BorderSide>(side),
      shape: ButtonStyleButton.allOrNull<OutlinedBorder>(shape),
      visualDensity: visualDensity,
      tapTargetSize: tapTargetSize,
      animationDuration: animationDuration,
      enableFeedback: enableFeedback,
      alignment: alignment,
      splashFactory: splashFactory,
    );
  }
}

/// 文本按钮 from Material
class FTextButton extends TextButton implements FActiveWidget {
  FTextButton({
    Key? key,
    required FWidget child,
    required VoidCallback? onPressed,
    required this.businessParent,
    required this.name,
    ButtonStyle? style,
  }) : super(
          key: key,
          child: child,
          onPressed: onPressed == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, (name ?? '') + " start");
                  onPressed.call();
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, (name ?? '') + " end");
                },
          style: style,
        );

  @override
  final FInteractiveContainer businessParent;

  @override
  final String? name;
}

/// 图标按钮 from Material
class FIconButton extends IconButton implements FActiveWidget {
  FIconButton({
    Key? key,
    double iconSize = 24.0,
    EdgeInsetsGeometry padding = const EdgeInsets.all(8.0),
    AlignmentGeometry alignment = Alignment.center,
    double? splashRadius,
    required FWidget icon,
    Color? color,
    Color? focusColor,
    Color? hoverColor,
    Color? highlightColor,
    Color? splashColor,
    Color? disabledColor,
    required VoidCallback onPressed,
    required this.businessParent,
    required this.name,
    MouseCursor mouseCursor = SystemMouseCursors.click,
  }) : super(
          key: key,
          icon: icon,
          onPressed: () {
            FActiveWidget.logActiveEvents(
                businessParent.pageName, (name ?? '') + " start");
            onPressed.call();
            FActiveWidget.logActiveEvents(
                businessParent.pageName, (name ?? '') + " end");
          },
          iconSize: iconSize,
          padding: padding,
          alignment: alignment,
          splashRadius: splashRadius,
          color: color,
          focusColor: focusColor,
          hoverColor: hoverColor,
          highlightColor: highlightColor,
          splashColor: splashColor,
          disabledColor: disabledColor,
          mouseCursor: mouseCursor,
        );

  @override
  final FInteractiveContainer businessParent;

  @override
  final String? name;
}

class FElevatedButtonWithIcon extends FElevatedButton {
  FElevatedButtonWithIcon({
    Key? key,
    required VoidCallback? onPressed,
    VoidCallback? onLongPress,
    ValueChanged<bool>? onHover,
    ValueChanged<bool>? onFocusChange,
    ButtonStyle? style,
    FocusNode? focusNode,
    bool? autofocus,
    Clip? clipBehavior,
    required this.icon,
    required this.label,
    required FInteractiveContainer businessParent,
    required String name,
  }) : super(
          key: key,
          onPressed: onPressed,
          onLongPress: onLongPress,
          onHover: onHover,
          onFocusChange: onFocusChange,
          style: style,
          focusNode: focusNode,
          autofocus: autofocus ?? false,
          clipBehavior: clipBehavior ?? Clip.none,
          child: _FElevatedButtonWithIconChild(icon: icon, label: label),
          businessParent: businessParent,
          name: name,
        );
  final Widget icon;
  final Widget label;

  @override
  ButtonStyle defaultStyleOf(BuildContext context) {
    final EdgeInsetsGeometry scaledPadding = ButtonStyleButton.scaledPadding(
      const EdgeInsetsDirectional.fromSTEB(12, 0, 16, 0),
      const EdgeInsets.symmetric(horizontal: 8),
      const EdgeInsetsDirectional.fromSTEB(8, 0, 4, 0),
      MediaQuery.maybeOf(context)?.textScaleFactor ?? 1,
    );
    return super.defaultStyleOf(context).copyWith(
          padding: MaterialStateProperty.all<EdgeInsetsGeometry>(scaledPadding),
        );
  }
}

class _FElevatedButtonWithIconChild extends StatelessWidget implements FWidget {
  const _FElevatedButtonWithIconChild({
    Key? key,
    required this.label,
    required this.icon,
  }) : super(key: key);

  final Widget label;
  final Widget icon;

  @override
  Widget build(BuildContext context) {
    final double scale = MediaQuery.maybeOf(context)?.textScaleFactor ?? 1;
    final double gap =
        scale <= 1 ? 8 : lerpDouble(8, 4, math.min(scale - 1, 1))!;
    return Row(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[icon, SizedBox(width: gap), Flexible(child: label)],
    );
  }
}

/// 延迟触发按钮
class FDelayedTiriggerButton extends StatefulWidget implements FWidget {
  FDelayedTiriggerButton({
    Key? key,
    required this.child,
    required this.onPressed,
    required this.businessParent,
    required this.name,
    this.style,
    this.countdownSeconds,
    VoidCallback? onLongPress,
    ValueChanged<bool>? onHover,
    ValueChanged<bool>? onFocusChange,
    FocusNode? focusNode,
    bool autofocus = false,
    Clip clipBehavior = Clip.none,
  });

  FInteractiveContainer businessParent;
  String? name;
  double? countdownSeconds = 5;
  VoidCallback? onPressed;
  Widget child;
  ButtonStyle? style;

  @override
  State<FDelayedTiriggerButton> createState() => _FDelayElevatedButtonState();
}

class _FDelayElevatedButtonState extends State<FDelayedTiriggerButton> {
  late double _countdownSeconds = widget.countdownSeconds ?? 5;
  Timer? _timer;

  @override
  void initState() {
    super.initState();
    startCountdown();
  }

  void startCountdown() {
    _timer = Timer.periodic(Duration(milliseconds: 100), (timer) {
      setState(() {
        if (_countdownSeconds > 0) {
          _countdownSeconds -= 0.1;
        } else {
          _timer?.cancel();
        }
      });
    });
  }

  @override
  void dispose() {
    _timer?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: ElevatedButton(
        onPressed: _countdownSeconds > 0 ? null : widget.onPressed,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            widget.child,
            if (_countdownSeconds > 0)
              Text(
                ' (' + (_countdownSeconds.toInt() + 1).toString() + ')',
                style: TextStyle(
                  color: Colors.black,
                  fontSize: 14,
                  height: 1.4,
                  fontFamily: FTheme.ins.localeSetting.fontFamily,
                ),
              ),
          ],
        ),
        style: widget.style?.copyWith(
          textStyle: MaterialStateProperty.all(
            TextStyle(
              color: _countdownSeconds > 0 ? Colors.black : Colors.white,
              fontSize: 16,
              fontFamily: FTheme.ins.localeSetting.fontFamily,
            ),
          ),
        ),
      ),
    );
  }
}
