import 'package:fis_ui/define.dart';
import 'package:flutter/material.dart';

/// 建立一个 [ActionDispatcher] 和一个 [Intent] 映射到
/// [Action] 在调用 [Action] 时由其后代使用。
///
/// 通常使用带有上下文的 [Actions.invoke] 调用操作
/// 包含环境 [Actions] 小部件。
///
/// {@工具飞镖}
/// 此示例创建一个自定义 [Action] 子类 `ModifyAction` 用于修改
/// 一个模型，另一个用于保存它的“SaveAction”。
///
/// 此示例演示将参数传递给要携带的 [Intent]
/// 那个行动]。动作可以从它们自己的构造中获取数据（比如
/// 本例中的 `model`)，或者来自调用时传递给它们的意图
/// （就像这个例子中的增量`amount`）。
///
/// 此示例还演示了如何使用 Intents 来限制小部件的
/// 对周围环境的依赖。在此定义的“SaveButton”小部件
/// example 可以调用在其祖先部件中定义的动作，它可以是
/// 定制以匹配它所在的小部件树的部分。它不
/// 需要了解 `SaveAction` 类，只有 `SaveIntent`，它
/// 只需要知道一个值通知器，而不是整个模型。
///
/// **请参阅 examples/api/lib/widgets/actions/actions.0.dart 中的代码 **
/// {@结束工具}
///
/// 也可以看看：
///
///  *[ActionDispatcher]，这个小部件用来管理动作的对象。
///  *[Action]，一个包含和定义用户调用的类
///    行动。
///  *[Intent]，一个拥有唯一的 [LocalKey] 标识动作的类，
///    以及运行 [Action] 的配置信息。
///  *[快捷方式]，一个用于将组合键绑定到 [Intent] 的小部件。
class FActions extends Actions implements FWidget {
  const FActions({
    Key? key,
    required Map<Type, Action<Intent>> actions,
    required FWidget child,
  }) : super(
          key: key,
          actions: actions,
          child: child,
        );
}
