import 'package:fis_ui/define.dart';
import 'package:fis_ui/interface/active_widget.dart';
import 'package:fis_ui/interface/interactive_container.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/widgets.dart';

/// 手势检测组件
class FGestureDetector extends GestureDetector implements FActiveWidget {
  @override
  final String name;

  @override
  final FInteractiveContainer businessParent;

  /// 手势检测组件
  ///
  /// [child] 子组件
  ///
  /// 各事件回调 参考 [GestureDetector]
  FGestureDetector({
    Key? key,
    required this.businessParent,
    required this.name,
    FWidget? child,
    GestureTapDownCallback? onTapDown,
    GestureTapUpCallback? onTapUp,
    GestureTapCallback? onTap,
    GestureTapCancelCallback? onTapCancel,
    GestureTapCallback? onSecondaryTap,
    GestureTapDownCallback? onSecondaryTapDown,
    GestureTapUpCallback? onSecondaryTapUp,
    GestureTapCancelCallback? onSecondaryTapCancel,
    GestureTapDownCallback? onTertiaryTapDown,
    GestureTapUpCallback? onTertiaryTapUp,
    GestureTapCancelCallback? onTertiaryTapCancel,
    GestureTapDownCallback? onDoubleTapDown,
    GestureTapCallback? onDoubleTap,
    GestureTapCancelCallback? onDoubleTapCancel,
    GestureLongPressCallback? onLongPress,
    GestureLongPressStartCallback? onLongPressStart,
    GestureLongPressMoveUpdateCallback? onLongPressMoveUpdate,
    GestureLongPressUpCallback? onLongPressUp,
    GestureLongPressEndCallback? onLongPressEnd,
    GestureLongPressCallback? onSecondaryLongPress,
    GestureLongPressStartCallback? onSecondaryLongPressStart,
    GestureLongPressMoveUpdateCallback? onSecondaryLongPressMoveUpdate,
    GestureLongPressUpCallback? onSecondaryLongPressUp,
    GestureLongPressEndCallback? onSecondaryLongPressEnd,
    GestureDragDownCallback? onVerticalDragDown,
    GestureDragStartCallback? onVerticalDragStart,
    GestureDragUpdateCallback? onVerticalDragUpdate,
    GestureDragEndCallback? onVerticalDragEnd,
    GestureDragCancelCallback? onVerticalDragCancel,
    GestureDragDownCallback? onHorizontalDragDown,
    GestureDragStartCallback? onHorizontalDragStart,
    GestureDragUpdateCallback? onHorizontalDragUpdate,
    GestureDragEndCallback? onHorizontalDragEnd,
    GestureDragCancelCallback? onHorizontalDragCancel,
    GestureDragDownCallback? onPanDown,
    GestureDragStartCallback? onPanStart,
    GestureDragUpdateCallback? onPanUpdate,
    GestureDragEndCallback? onPanEnd,
    GestureDragCancelCallback? onPanCancel,
    GestureScaleStartCallback? onScaleStart,
    GestureScaleUpdateCallback? onScaleUpdate,
    GestureScaleEndCallback? onScaleEnd,
    GestureForcePressStartCallback? onForcePressStart,
    GestureForcePressPeakCallback? onForcePressPeak,
    GestureForcePressUpdateCallback? onForcePressUpdate,
    GestureForcePressEndCallback? onForcePressEnd,
    HitTestBehavior? behavior,
    bool excludeFromSemantics = false,
    DragStartBehavior dragStartBehavior = DragStartBehavior.start,
  }) : super(
          key: key,
          child: child,
          onTapDown: onTapDown == null
              ? null
              : (_d) {
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onTapDown start");
                  onTapDown.call(_d);
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onTapDown end");
                },
          onTapUp: onTapUp == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onTapUp start");
                  onTapUp.call(details);
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onTapUp end");
                },
          onTap: onTap == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onTap start");
                  onTap.call();
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onTap end");
                },
          onTapCancel: onTapCancel == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onTapCancel start");
                  onTapCancel.call();
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onTapCancel end");
                },
          onSecondaryTap: onSecondaryTap == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryTap start");
                  onSecondaryTap.call();
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryTap end");
                },
          onSecondaryTapDown: onSecondaryTapDown == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryTapDown start");
                  onSecondaryTapDown(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryTapDown end");
                },
          onSecondaryTapUp: onSecondaryTapUp == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryTapUp start");
                  onSecondaryTapUp.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryTapUp end");
                },
          onSecondaryTapCancel: onSecondaryTapCancel == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryTapCancel start");
                  onSecondaryTapCancel.call();
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryTapCancel end");
                },
          onTertiaryTapDown: onTertiaryTapDown == null
              ? null
              : (_d) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onTertiaryTapDown start");
                  onTertiaryTapDown.call(_d);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onTertiaryTapDown end");
                },
          onTertiaryTapUp: onTertiaryTapUp == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onTertiaryTapUp start");
                  onTertiaryTapUp.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onTertiaryTapUp end");
                },
          onTertiaryTapCancel: onTertiaryTapCancel == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onTertiaryTapCancel start");
                  onTertiaryTapCancel.call();
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onTertiaryTapCancel end");
                },
          onDoubleTapDown: onDoubleTapDown == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onDoubleTapDown start");
                  onDoubleTapDown.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onDoubleTapDown end");
                },
          onDoubleTap: onDoubleTap == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onDoubleTap start");
                  onDoubleTap.call();
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onDoubleTap end");
                },
          onDoubleTapCancel: onDoubleTapCancel == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onDoubleTapCancel start");
                  onDoubleTapCancel.call();
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onDoubleTapCancel end");
                },
          onLongPress: onLongPress == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onLongPress start");
                  onLongPress.call();
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onLongPress end");
                },
          onLongPressStart: onLongPressStart == null
              ? null
              : (_d) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onLongPressStart start");
                  onLongPressStart.call(_d);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onLongPressStart end");
                },
          onLongPressMoveUpdate: onLongPressMoveUpdate == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onLongPressMoveUpdate start");
                  onLongPressMoveUpdate.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onLongPressMoveUpdate end");
                },
          onLongPressUp: onLongPressUp == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onLongPressUp start");
                  onLongPressUp.call();
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onLongPressUp end");
                },
          onLongPressEnd: onLongPressEnd == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onLongPressEnd start");
                  onLongPressEnd.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onLongPressEnd end");
                },
          onSecondaryLongPress: onSecondaryLongPress == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onLongPressEnd start");
                  onSecondaryLongPress.call();
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onLongPressEnd end");
                },
          onSecondaryLongPressStart: onSecondaryLongPressStart == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryLongPressStart start");
                  onSecondaryLongPressStart.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryLongPressStart end");
                },
          onSecondaryLongPressMoveUpdate: onSecondaryLongPressMoveUpdate == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryLongPressStart start");
                  onSecondaryLongPressMoveUpdate.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryLongPressStart end");
                },
          onSecondaryLongPressUp: onSecondaryLongPressUp == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryLongPressUp start");
                  onSecondaryLongPressUp.call();
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryLongPressUp end");
                },
          onSecondaryLongPressEnd: onSecondaryLongPressEnd == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryLongPressEnd start");
                  onSecondaryLongPressEnd.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onSecondaryLongPressEnd end");
                },
          onVerticalDragDown: onVerticalDragDown == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onVerticalDragDown start");
                  onVerticalDragDown.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onVerticalDragDown end");
                },
          onVerticalDragStart: onVerticalDragStart == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onVerticalDragStart start");
                  onVerticalDragStart.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onVerticalDragStart end");
                },
          onVerticalDragUpdate: onVerticalDragUpdate == null
              ? null
              : ((details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onVerticalDragUpdate start");
                  onVerticalDragUpdate.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onVerticalDragUpdate end");
                }),
          onVerticalDragEnd: onVerticalDragEnd == null
              ? null
              : ((details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onVerticalDragEnd start");
                  onVerticalDragEnd.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onVerticalDragEnd end");
                }),
          onVerticalDragCancel: onVerticalDragCancel == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onVerticalDragEnd start");
                  onVerticalDragCancel.call();
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onVerticalDragEnd end");
                },
          onHorizontalDragDown: onHorizontalDragDown == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onHorizontalDragDown start");
                  onHorizontalDragDown.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onHorizontalDragDown end");
                },
          onHorizontalDragStart: onHorizontalDragStart == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onHorizontalDragStart start");
                  onHorizontalDragStart.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onHorizontalDragStart end");
                },
          onHorizontalDragUpdate: onHorizontalDragUpdate == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onHorizontalDragUpdate start");
                  onHorizontalDragUpdate.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onHorizontalDragUpdate end");
                },
          onHorizontalDragEnd: onHorizontalDragEnd == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onHorizontalDragEnd start");
                  onHorizontalDragEnd.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onHorizontalDragEnd end");
                },
          onHorizontalDragCancel: onHorizontalDragCancel == null
              ? null
              : () {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onHorizontalDragCancel start");
                  onHorizontalDragCancel.call();
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onHorizontalDragCancel end");
                },
          onPanDown: onPanDown == null
              ? null
              : ((details) {
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onPanDown start");
                  onPanDown.call(details);
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onPanDown end");
                }),
          onPanStart: onPanStart == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onPanStart start");
                  onPanStart.call(details);
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onPanStart end");
                },
          onPanUpdate: onPanUpdate == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onPanStart start");
                  onPanUpdate.call(details);
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onPanStart end");
                },
          onPanEnd: onPanEnd == null
              ? null
              : (d) {
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onPanEnd start");
                  onPanEnd.call(d);
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onPanEnd end");
                },
          onPanCancel: onPanCancel == null
              ? null
              : (() {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onPanCancel start");
                  onPanCancel.call();
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onPanCancel end");
                }),
          onScaleStart: onScaleStart == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onScaleStart start");
                  onScaleStart.call(details);
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onScaleStart end");
                },
          onScaleUpdate: onScaleUpdate == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onScaleUpdate start");
                  onScaleUpdate.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onScaleUpdate end");
                },
          onScaleEnd: onScaleEnd == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onScaleEnd start");
                  onScaleEnd.call(details);
                  FActiveWidget.logActiveEvents(
                      businessParent.pageName, name + "/" + "onScaleEnd end");
                },
          onForcePressStart: onForcePressStart == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onForcePressStart start");
                  onForcePressStart.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onForcePressStart end");
                },
          onForcePressPeak: onForcePressPeak == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onForcePressPeak start");
                  onForcePressPeak.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onForcePressPeak end");
                },
          onForcePressUpdate: onForcePressUpdate == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onForcePressUpdate start");
                  onForcePressUpdate.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onForcePressUpdate end");
                },
          onForcePressEnd: onForcePressEnd == null
              ? null
              : (details) {
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onForcePressEnd start");
                  onForcePressEnd.call(details);
                  FActiveWidget.logActiveEvents(businessParent.pageName,
                      name + "/" + "onForcePressEnd end");
                },
          behavior: behavior,
          excludeFromSemantics: excludeFromSemantics,
          dragStartBehavior: dragStartBehavior,
        );
}
