import 'package:fis_ui/define.dart';
import 'package:flutter/material.dart';

/// 弹性布局
class FFlex extends Flex implements FWidget {
  /// 弹性布局
  ///
  /// [direction] 布局方向, 默认水平方向
  ///
  /// [textDirection] 排列方向
  ///
  /// [textBaseline] 对齐基线
  ///
  /// [mainAxisAlignment] 主轴对齐方式
  ///
  /// [mainAxisSize] 主轴占用空间
  ///
  /// [crossAxisAlignment] 交叉轴对齐方式
  ///
  /// [verticalDirection] 垂直排列方向
  ///
  /// [clipBehavior] 裁剪行为
  FFlex({
    Key? key,
    Axis direction = Axis.horizontal,
    TextDirection textDirection = TextDirection.ltr,
    TextBaseline textBaseline = TextBaseline.alphabetic,
    MainAxisAlignment mainAxisAlignment = MainAxisAlignment.start,
    MainAxisSize mainAxisSize = MainAxisSize.max,
    CrossAxisAlignment crossAxisAlignment = CrossAxisAlignment.center,
    VerticalDirection verticalDirection = VerticalDirection.down,
    Clip clipBehavior = Clip.none,
    List<FWidget> children = const <FWidget>[],
  }) : super(
          key: key,
          direction: direction,
          textDirection: textDirection,
          textBaseline: textBaseline,
          mainAxisAlignment: mainAxisAlignment,
          mainAxisSize: mainAxisSize,
          crossAxisAlignment: crossAxisAlignment,
          verticalDirection: verticalDirection,
          clipBehavior: clipBehavior,
          children: children,
        );
}

/// 控制[Row]、[Column]或[Flex]的子组件如何伸缩的组件
class FFlexible extends Flexible implements FWidget {
  /// 控制[Row]、[Column]或[Flex]的子控件如何伸缩的组件
  ///
  /// [child] 子组件
  ///
  /// [flex] 弹性因子
  ///
  /// [fit] 填充方式
  const FFlexible({
    required FWidget child,
    Key? key,
    int flex = 1,
    FlexFit fit = FlexFit.loose,
  }) : super(
          key: key,
          child: child,
          fit: fit,
          flex: flex,
        );
}

/// 展开[Row]、[Column]或[Flex]子组件的组件，使其子组件填满可用的空间
class FExpanded extends FFlexible implements FWidget {
  /// 展开[Row]、[Column]或[Flex]子组件的组件，使其子组件填满可用的空间
  ///
  /// [child] 子组件
  ///
  /// [flex] 弹性因子
  const FExpanded({
    required FWidget child,
    Key? key,
    int flex = 1,
  }) : super(
          key: key,
          child: child,
          flex: flex,
          fit: FlexFit.tight,
        );
}
