import 'package:fis_ui/define.dart';
import 'package:flutter/material.dart';

/// 堆叠布局
class FStack extends Stack implements FWidget {
  /// 堆叠布局
  ///
  /// [children] 子组件集合
  ///
  /// [textDirection] 排列方向
  ///
  /// [alignment] 对齐方式
  ///
  /// [fit] 约束方式
  ///
  /// [clipBehavior] 裁剪行为
  FStack({
    Key? key,
    TextDirection? textDirection,
    AlignmentGeometry alignment = AlignmentDirectional.topStart,
    StackFit fit = StackFit.loose,
    Clip clipBehavior = Clip.hardEdge,
    List<FWidget> children = const <FWidget>[],
  }) : super(
          key: key,
          textDirection: textDirection,
          alignment: alignment,
          fit: fit,
          clipBehavior: clipBehavior,
          children: children,
        );
}

/// 根据索引排列的堆叠布局
class FIndexedStack extends IndexedStack implements FWidget {
  FIndexedStack({
    Key? key,
    TextDirection? textDirection,
    AlignmentGeometry alignment = AlignmentDirectional.topStart,
    int index = 0,
    StackFit sizing = StackFit.loose,
    List<FWidget> children = const <FWidget>[],
  }) : super(
          key: key,
          textDirection: textDirection,
          alignment: alignment,
          index: index,
          sizing: sizing,
          children: children,
        );
}

/// 用于控制[FStack]的子组件的位置。
class FPositioned extends Positioned implements FWidget {
  const FPositioned({
    Key? key,
    double? left,
    double? top,
    double? right,
    double? bottom,
    double? width,
    double? height,
    required FWidget child,
  }) : super(
          key: key,
          child: child,
          left: left,
          right: right,
          top: top,
          bottom: bottom,
          width: width,
          height: height,
        );
}
