import 'package:fis_ui/define.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

/// 路由页面头部导航栏
class FAppBar extends AppBar implements FPreferredSizeWidget {
  /// 路由页面头部导航栏
  ///
  /// [leading] 最左侧栏位
  ///
  /// [leadingWidth] 最左侧栏位占用宽度
  ///
  /// [title] 标题栏位
  ///
  /// [titleSpacing] 标题栏位
  ///
  /// [elevation] 阴影宽度
  ///
  /// [shadowColor] 阴影颜色
  ///
  /// [centerTitle] 标题是否居中
  ///
  /// [actions] 最右侧菜单
  ///
  /// [backgroundColor] 背景色
  ///
  /// [foregroundColor] 前景色
  FAppBar({
    Key? key,
    FWidget? leading,
    double? leadingWidth,
    FWidget? title,
    double? titleSpacing,
    double? elevation,
    Color? shadowColor,
    bool? centerTitle,
    List<FWidget>? actions,
    Color? backgroundColor,
    Color? foregroundColor,
    bool automaticallyImplyLeading = true,
    FWidget? flexibleSpace,
    FPreferredSizeWidget? bottom,
    ShapeBorder? shape,
    IconThemeData? iconTheme,
    IconThemeData? actionsIconTheme,
    bool primary = true,
    bool excludeHeaderSemantics = false,
    double toolbarOpacity = 1.0,
    double bottomOpacity = 1.0,
    double? toolbarHeight,
    TextStyle? toolbarTextStyle,
    TextStyle? titleTextStyle,
    SystemUiOverlayStyle? systemOverlayStyle,
  }) : super(
          key: key,
          leading: leading,
          leadingWidth: leadingWidth,
          title: title,
          titleSpacing: titleSpacing,
          centerTitle: centerTitle,
          elevation: elevation,
          shadowColor: shadowColor,
          actions: actions,
          backgroundColor: backgroundColor,
          foregroundColor: foregroundColor,
          automaticallyImplyLeading: automaticallyImplyLeading,
          flexibleSpace: flexibleSpace,
          bottom: bottom,
          shape: shape,
          iconTheme: iconTheme,
          actionsIconTheme: actionsIconTheme,
          primary: primary,
          excludeHeaderSemantics: excludeHeaderSemantics,
          toolbarOpacity: toolbarOpacity,
          bottomOpacity: bottomOpacity,
          toolbarHeight: toolbarHeight,
          toolbarTextStyle: toolbarTextStyle,
          titleTextStyle: titleTextStyle,
          systemOverlayStyle: systemOverlayStyle,
        );

  /// 简单顶部条
  ///
  /// [title] 标题
  ///
  /// [onBackClicked] 回退点击事件
  static FPreferredSizeWidget simple({
    Key? key,
    String? title,
    VoidCallback? onBackClicked,
    List<FWidget>? actions,
  }) =>
      _SimpleAppbar(
        key: key,
        title: title,
        onBackClicked: onBackClicked,
        actions: actions,
      );
}

class _SimpleAppbar extends StatelessWidget implements FPreferredSizeWidget {
  _SimpleAppbar({
    Key? key,
    this.title,
    this.onBackClicked,
    this.actions,
  }) : super(key: key);

  final String? title;
  final VoidCallback? onBackClicked;
  final List<FWidget>? actions;

  @override
  Widget build(BuildContext context) {
    final titleWidget =
        title != null && title!.isNotEmpty ? Text(title!) : null;
    return AppBar(
      key: key,
      // foregroundColor: Colors.black87,
      // backgroundColor: Colors.white,
      title: titleWidget,
      centerTitle: true,
      elevation: 0,
      leading: IconButton(
        icon: Icon(Icons.arrow_back_ios_new),
        onPressed: onBackClicked,
      ),
      actions: actions,
    );
  }

  @override
  Size get preferredSize => Size.fromHeight(kToolbarHeight);
}

abstract class FPreferredSizeWidget extends PreferredSizeWidget
    implements FWidget {}

class FPreferredSize extends PreferredSize implements FPreferredSizeWidget {
  FPreferredSize({
    Key? key,
    required FWidget child,
    required Size preferredSize,
  }) : super(
          key: key,
          child: child,
          preferredSize: preferredSize,
        );
}
