import 'package:flutter/material.dart';

/// 滚动控制器扩展
extension FScrollControllerExtensions on ScrollController {
  static final _scrollToBottomFnMap = <int, VoidCallback?>{};

  /// 滚动到底部回调
  VoidCallback? get onScrollToBottom {
    return _scrollToBottomFnMap[this.hashCode];
  }

  set onScrollToBottom(VoidCallback? fn) {
    if (fn == null) {
      _cancelListenScrollToBottom();
    } else {
      _listenScrollToBottom(fn);
    }
  }

  _listenScrollToBottom(VoidCallback callback) {
    final hasListened = _scrollToBottomFnMap.containsKey(this.hashCode);
    if (!hasListened) {
      this.addListener(_onCheckScrollToBottom);
    }
    _scrollToBottomFnMap[this.hashCode] = callback;
  }

  _cancelListenScrollToBottom() {
    this.removeListener(_onCheckScrollToBottom);
    _scrollToBottomFnMap.remove(this.hashCode);
  }

  void _onCheckScrollToBottom() {
    final pos = this.position;
    if (pos.pixels == pos.maxScrollExtent) {
      final fn = _scrollToBottomFnMap[this.hashCode];
      fn?.call();
    }
  }
}
